/* SqueakEventQueue.m created by marcel on Wed 14-Feb-2001 */

#import "SqueakEventQueue.h"
#import "SqueakEvent.h"
#import "AccessorMacros.h"

@implementation SqueakEventQueue

intAccessor( inputEventSemaphore, setInputEventSemaphore)

-(void)inputDidArrive
{
    signalSemaphoreWithIndex( inputEventSemaphore );
}

-init
{
    self = [super init];
    lock = [[NSLock alloc] init];
    queue = [[NSMutableArray alloc] init];
    return self;
}

-(void)putEvent:(sqInputEvent*)anEvent atTime:(int)ms
{
    id event=[[SqueakEvent alloc] init];
    [lock lock];
    anEvent->timeStamp = ms;
    if ( anEvent->type == EventTypeMouse ) {
        sqMouseEvent *thisMouseEvent = (sqMouseEvent*)anEvent;
        //--- update event state info

        currentX = thisMouseEvent->x;
        currentY = thisMouseEvent->y;
        mouseButtonState = thisMouseEvent->buttons;
        if ( mouseButtonState != 0 ) {
            lastEventTime=ms;
        }
        if ( [queue count] ) {
            sqMouseEvent *lastMouseEvent = (sqMouseEvent*)[[queue lastObject] eventData];

            //--- merge mouse-moved
            if ( (lastMouseEvent->type == thisMouseEvent->type) &&  (thisMouseEvent->type== EventTypeMouse) &&
                 (lastMouseEvent->buttons == thisMouseEvent->buttons) && (thisMouseEvent->buttons== 0) ) {
                [queue removeLastObject];
            }
       }

        if ( [queue count] > 8 ) {
            [queue removeObjectAtIndex:0];
        }
    }
    *[event eventData]=*anEvent;
    [queue addObject:event];
    [lock unlock];
    [event release];
}

-(BOOL)getEvent:(sqInputEvent*)evt
{
    evt->type=EventTypeNone;
    [lock lock];
    if ( [queue count] ) {
        *evt=*[[queue objectAtIndex:0] eventData];
        [queue removeObjectAtIndex:0];
    }
    [lock unlock];
    return evt->type != EventTypeNone;
}

-(int)buttonState
{
//   NSLog(@"mouse: %x keyboard : %x together: %x",mouseButtonState,keyboardModifierState,
//          (keyboardModifierState << 3) | mouseButtonState);
    return (keyboardModifierState << 3) | mouseButtonState;
}

-(int)squeakCodedMouseLocation
{
    return(((currentX&0xffff)<<16)|(currentY&0xffff));
}

-(int)_getOrPeekKeyStroke:(BOOL)doGet
{
    int key=-1;
    [lock lock];
    while ( [queue count] )
    {
        sqKeyboardEvent *evt=(sqKeyboardEvent*)[[queue objectAtIndex:0] eventData];
        if ( evt->type == EventTypeKeyboard ) {
            key=evt->charCode;
            keyboardModifierState=evt->modifiers;
            if ( doGet ) {
                [queue removeObjectAtIndex:0];	//	discard the key event we just got
            }
            break;
        } else {
            [queue removeObjectAtIndex:0];	//	discard non key events
        }
    }
    [lock unlock];
    return key;
}

-(int)peekKeyStroke
{
    return [self _getOrPeekKeyStroke:NO];
}


-(int)getKeyStroke
{
    return [self _getOrPeekKeyStroke:YES];
}

-(int)lastEventTime
{
    return lastEventTime; 
}

-(void)dealloc
{
    [lock release];
    [queue release];
    [super dealloc];
}

@end
