/* SqueakImageFile.m created by marcel on Wed 27-Dec-2000 */

#import "SqueakImageFile.h"
#import "AccessorMacros.h"

#define sqImageFile id

@implementation SqueakImageFile

idAccessor( fileName, setFileName )
idAccessor( mode, setMode )
idAccessor( data, setData )

+imageFileWithName:(NSString*)name mode:(NSString*)newMode
{
    return [[self alloc] initWithContentsOfFile:name mode:newMode];
}

-initWithContentsOfFile:(NSString*)newFileName mode:(NSString*)newMode
{
    id pool=[[NSAutoreleasePool alloc] init];
    self = [super init];
    [self setFileName:newFileName];
    [self setMode:newMode];
    if ( [mode rangeOfString:@"r"].length>0 ) {
        [self setData:[NSMutableData dataWithContentsOfMappedFile:newFileName]];
    } else {
        [self setData:[NSMutableData data]];
    }
    [pool release];
    if ( !data ) {
        [self release];
        return nil;
    }
    return self;
}

-(void)write
{
    id pool=[[NSAutoreleasePool alloc] init];
    [data writeToFile:[self fileName] atomically:YES];
    dirty=NO;
    [pool release];
}

-(void)close
{
    if ( dirty && [mode rangeOfString:@"w"].length>0 ) {
        [self write];
    }
    [self release];
}

-(int)position
{
    return offset;
}

-(int)read:(void*)ptr length:(int)length
{
    [data getBytes:ptr range:NSMakeRange( offset, length)];
    offset+=length;
    return length;
}

-(void)seek:(int)pos
{
    offset=pos;
}

-(int)write:(void*)ptr length:(int)length
{
    dirty=YES;
    if ( offset < [data length] ) {
        [data replaceBytesInRange:NSMakeRange(offset,length) withBytes:ptr];
    } else {
        [data setLength:offset];
        [data appendBytes:ptr length:length];
    }
    offset+=length;
    return length;
}

-(void)dealloc
{
    [fileName release];
    [mode release];
    [data release];
    [super dealloc];
}

@end

void        sqImageFileClose(sqImageFile f)
{
    [(id)f close];
}

sqImageFile sqImageFileOpen(char *fileName, char *mode)
{
    id pool =[[NSAutoreleasePool alloc] init];
    id name=[NSString stringWithCString:fileName];
    id modeString = [NSString stringWithCString:mode];
    id file =[SqueakImageFile imageFileWithName:name mode:modeString];
    [pool release];
    return (sqImageFile)file;
}

int         sqImageFilePosition(sqImageFile f)
{
    return [(id)f position];
}

int         sqImageFileRead(void *ptr, int elementSize, int count, sqImageFile f)
{
    return [(id)f read:ptr length:elementSize*count] / elementSize;
}

void        sqImageFileSeek(sqImageFile f, int pos)
{
    [(id)f seek:pos];
}

int         sqImageFileWrite(void *ptr, int elementSize, int count, sqImageFile f)
{
    return [(id)f write:ptr length:elementSize * count] / elementSize;
}
