/* SqueakInterpreter.m created by marcel on Tue 19-Dec-2000 */

#import "SqueakInterpreter.h"
#import "NSStringSqueakString.h"
#import "sq.h"
#import "SqueakImageFile.h"
#import "SqueakEventQueue.h"
#import "SqueakInterpreterTimes.h"

@implementation SqueakInterpreter

extern int stackCacheEntries;
extern int contextCacheEntries;
intAccessor( stackCacheEntries, setStackCacheEntries )
intAccessor( contextCacheEntries, setContextCacheEntries )

idAccessor( display, setDisplay )
idAccessor( inputSensor, setInputSensor )
idAccessor( eventSensor, setEventSensor )
idAccessor( soundServer, setSoundServer )
idAccessor( imagePath, _setImagePath )
idAccessor( vmPath, setVmPath )
idAccessor( argvAttributes, setArgvAttributes )
idAccessor( specialAttributes, setSpecialAttributes )
idAccessor( clipHandler, setClipHandler )
boolAccessor( interpreterRunning , setInterpreterRunning )
idAccessor( eventQueue, setEventQueue )
idAccessor( squeakMemory, setSqueakMemory )
static id interpreter=nil;

+interpreter
{
    if ( !interpreter ) {
        interpreter=[[self alloc] init];
    }
    return interpreter;
}

-init
{
    self = [super init];
    eventQueue=[[SqueakEventQueue alloc] init];
    return self;
}

//---	global var required by interpreter, so have to keep in sync

char imageName[8192+1];
-(void)setImagePath:newName
{
    int len=MIN([newName length],8192);
    [self _setImagePath:newName];
    [newName getSqueakString:imageName maxLength:len];
    imageName[len]=0;
    if ( [[self eventSensor] respondsToSelector:@selector(setDefaultImagePath:)] ) {
        [[self eventSensor] setDefaultImagePath:newName];
    }
}

-(void)verifySizes
{
    if (sizeof(int) != 4){
        [self terminateWithCompilerAlertMessage:@"integers are not 32 bits."];
    }
    if (sizeof(double) != 8){
        [self terminateWithCompilerAlertMessage:@"doubles are not 64 bits."];
    }
    if (sizeof(time_t) != 4){
        [self terminateWithCompilerAlertMessage:@" time_t's are not 32 bits."];
    }
}

-(BOOL)readImageFile:(NSString*)newImagePath withHeapSize:(int)heapSize
{
    id pool=[[NSAutoreleasePool alloc] init];
    id imageFile=[SqueakImageFile imageFileWithName:newImagePath mode:@"r"];
    if ( imageFile ) {
        [self setImagePath:newImagePath];
        readImageFromFileHeapSize( imageFile, heapSize );
        [imageFile close];
    }
    [pool release];
    return imageFile != nil;
}

extern int squeakInterpretNotDone;
-(void)stopInterpreter
{
    squeakInterpretNotDone=NO;
}

-(void)tenthOfASecondTick
{
    lowResMSTime = [self _currentTimeInterval];
}

-(void)startInterpreter
{
    id pool=[[NSAutoreleasePool alloc] init];
    startupTimeInterval = [self _currentTimeInterval];
    lowResMSTime = startupTimeInterval;
//    [self startTicking];
    sqFileInit();
    joystickInit();
    [pool release];

    interpreterRunning=YES;
    interpret();
    interpreterRunning=NO;
}

-(void)startInterpreterInSeparateThread
{
    if (!interpreterRunning) {
        interpreterRunning=YES;
        timer = [[NSTimer scheduledTimerWithTimeInterval:.01 target:self selector:@selector(tenthOfASecondTick) userInfo:nil repeats:YES] retain];

        [NSThread detachNewThreadSelector:@selector(startInterpreter) toTarget:self withObject:nil];
    } else {
        [NSException raise:@"already-started" format:@"interpreter is running already"];
    }
}

-(void)interrupt
{
    extern int interruptPending;
    extern int interruptCheckCounter;
    interruptPending = YES;
    interruptCheckCounter=0;
}

-attributeAtIndex:(int)attributeIndex
{
    id relevantArray=nil;
    id result=@"";
    if ( attributeIndex < 0 ) {
        relevantArray=specialAttributes;
        attributeIndex=-attributeIndex;
    } else {
        relevantArray=argvAttributes;
        attributeIndex=attributeIndex+1;
    }
    if ( [relevantArray count] > attributeIndex ) {
        result =  [relevantArray objectAtIndex:attributeIndex];
    }
//    NSLog(@"attribute %d = %@",attributeIndex,result);
    return result;
}

-(int)attributeLengthAtIndex:(int)attributeIndex
{
    return [[self attributeAtIndex:attributeIndex] length];
}

-(void*)sqAllocateMemory:(int)heapSize
{
    if ( !squeakMemory ) {
        squeakMemory=[[NSMutableData alloc] initWithLength:heapSize+64];
    }
    return [squeakMemory mutableBytes];
}

@end

int imageNameSize(void)
{
    return [[[SqueakInterpreter interpreter] imagePath] length];
}

int imageNameGetLength(int sqImageNameIndex,int length)
{
    [[[SqueakInterpreter interpreter] imagePath] getSqueakString:(char*)sqImageNameIndex maxLength:length];
    return imageNameSize();
};


int imageNamePutLength(int sqImageNameIndex,int length)
{
    [[SqueakInterpreter interpreter] setImagePath:[NSString stringWithSqueakString:(char*)sqImageNameIndex length:length]];
    return 0;
}

int vmPathSize(void)
{
    return [[[SqueakInterpreter interpreter] vmPath] length];
}

int vmPathGetLength(int sqVMPathIndex,int length)
{
    [[[SqueakInterpreter interpreter] vmPath] getSqueakString:(char*)sqVMPathIndex maxLength:length];
    return vmPathSize();
}


/* system attributes */
int attributeSize(int ident)
{
    return([[SqueakInterpreter interpreter] attributeLengthAtIndex:ident]);
}

int getAttributeIntoLength(int ident,int byteArrayIndex,int length)
{
    [[[SqueakInterpreter interpreter] attributeAtIndex:ident] getSqueakString:(char*)byteArrayIndex maxLength:length];
    return [[SqueakInterpreter interpreter] attributeLengthAtIndex:ident];
}

void *sqAllocateMemory( int minHeapSize, int desiredHeapSize )
{
    return [[SqueakInterpreter interpreter] sqAllocateMemory:desiredHeapSize];
}
