/* SqueakInterpreterEventSensor.m created by marcel on Wed 20-Dec-2000 */

#include <mach/mach.h>
#include <mach/message.h>
#import "SqueakInterpreterEventSensor.h"
#import "SqueakInterpreterTimes.h"
#import "sq.h"
#import "SqueakEventQueue.h"

@implementation SqueakInterpreter(EventSensor)

-(void)mouseAt:(NSPoint)location leftButton:(BOOL)leftButton middleButton:(BOOL)middleButton rightButton:(BOOL)rightButton
{
    sqInputEvent event;
    sqMouseEvent* mouseEvent = (sqMouseEvent*)&event;

    mouseEvent->type = EventTypeMouse;
    mouseEvent->x=location.x;
    mouseEvent->y=location.y;
    mouseEvent->buttons=0;
    if ( rightButton ) {
        mouseEvent->buttons |= BlueButtonBit;
    }
    if ( middleButton ) {
        mouseEvent->buttons |= YellowButtonBit;
    }
    if ( leftButton ) {
        mouseEvent->buttons |= RedButtonBit;
    }
    mouseEvent->modifiers= 0;
    if ( middleButton ) {
        mouseEvent->modifiers |= CommandKeyBit;
    }
    if ( rightButton ) {
        mouseEvent->modifiers |= OptionKeyBit;
   }
    [eventQueue putEvent:&event atTime:[self ioMSecs]];
}

-(void)keyDown:(int)charCode keyCode:(int)keyCode shift:(BOOL)shift control:(BOOL)control alternate:(BOOL)alternate command:(BOOL)command
{
    sqInputEvent event;
    sqKeyboardEvent *keyEvent=(sqKeyboardEvent*)&event;

    keyEvent->type = EventTypeKeyboard;
    keyEvent->pressCode = EventKeyChar;
    keyEvent->charCode = charCode;
    keyEvent->modifiers= 0;
    if ( shift ) {
        keyEvent->modifiers |= ShiftKeyBit;
    }
    if ( control ) {
        keyEvent->modifiers |= CtrlKeyBit;
    }
    if ( alternate ) {
        keyEvent->modifiers |= OptionKeyBit;
   }
    if ( command ) {
        keyEvent->modifiers |= CommandKeyBit;
    }
    [eventQueue putEvent:&event atTime:[self ioMSecs]];
}

-(int)ioBeep
{
    return 0;
}

-(void)ioExit
{
    id pool=[[NSAutoreleasePool alloc] init];
    [self stopInterpreter];
    [[self eventSensor] ioExit];
    [pool release];
}

-(int)ioProcessEvents:(int)microseconds
{
    aioPollForIO(microseconds,0);	/* quick check for asynchronous socket i/o */
    return 0;
}

-(int)ioRelinquishProcessorForMicroseconds:(int)microSeconds
{
    int microSecondsToSleep;
    int actualMicros;
    int tries=0;
    microSeconds = MAX(5000,microSeconds);
    microSecondsToSleep=microSeconds;
    if (  [self ioMSecs] - [eventQueue lastEventTime]  < 1000 ) {
        return 0;
    } else {
        NSAutoreleasePool* pool=[[NSAutoreleasePool alloc] init];
        NSTimeInterval time,start=[NSDate timeIntervalSinceReferenceDate];
        do {
            NSDate* sleepUntilDate=[NSDate dateWithTimeIntervalSinceNow:((double)microSecondsToSleep)/1000000.0];
            [NSThread sleepUntilDate:sleepUntilDate];
            [self ioProcessEvents:0];
            time = [NSDate timeIntervalSinceReferenceDate]-start;
            microSecondsToSleep = microSeconds - (time*1000000);
            tries++;
        } while ( microSecondsToSleep > 0 && tries<1 );
        actualMicros  =(int)(time*1000000);
        if ( NO && (microSeconds > 10000 || actualMicros > 30000) ) {
            NSLog(@"wanted to sleep for %d micro-s actually slept for %d micro-s, tries=%d",microSeconds,actualMicros,tries);
        }
        [pool release];
        return time*1000;
    }
}

@end

int ioSetInputSemaphore(int semaIndex)
{
    [[[SqueakInterpreter interpreter] eventQueue] setInputEventSemaphore:semaIndex];
    success(YES);
    return YES;
}

int ioGetNextEvent(sqInputEvent *evt)
{
    return [[[SqueakInterpreter interpreter] eventQueue] getEvent:evt];
}

int ioBeep(void)
{
    return([[SqueakInterpreter interpreter] ioBeep]);
}

int ioExit(void)
{
    [[SqueakInterpreter interpreter] ioExit];
    return 0;
}

int ioProcessEvents(void)
{
    return([[SqueakInterpreter interpreter] ioProcessEvents:0]);
}

int ioRelinquishProcessorForMicroseconds(int microSeconds)
{
#if 0
    return 0;
#else    
    int retval;
    retval = [[SqueakInterpreter interpreter]
                        ioRelinquishProcessorForMicroseconds:microSeconds];
    return retval;
#endif    
}

