/* sqUnixSound.c -- sound support via Open Sound System
 * 
 *   Copyright (C) 1996 1997 1998 1999 2000 Ian Piumarta and any individual
 *   authors/contributors listed elsewhere in this file.
 *   All rights reserved.
 *   
 *   This file is part of Unix Squeak.
 * 
 *   This file is distributed in the hope that it will be useful, but WITHOUT
 *   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *   FITNESS FOR A PARTICULAR PURPOSE.
 *   
 *   You may use and/or distribute this file ONLY as part of Squeak, under
 *   the terms of the Squeak License as described in `LICENSE' in the base of
 *   this distribution, subject to the following restrictions:
 * 
 *   1. The origin of this software must not be misrepresented; you must not
 *      claim that you wrote the original software.  If you use this software
 *      in a product, an acknowledgment to the original author(s) (and any
 *      other contributors mentioned herein) in the product documentation
 *      would be appreciated but is not required.
 * 
 *   2. This notice may not be removed or altered in any source distribution.
 * 
 *   Using or modifying this file for use in any context other than Squeak
 *   changes these copyright conditions.  Read the file `COPYING' in the base
 *   of the distribution before proceeding with any such use.
 * 
 *   You are STRONGLY DISCOURAGED from distributing a modified version of
 *   this file under its original name without permission.  If you must
 *   change it, rename it first.
 */

/* Author: Ian Piumarta (ian.piumarta@inria.fr)
 *
 * Last edited: Fri Aug 25 23:53:29 2000 by piumarta (Ian Piumarta) on tif
 *
 * Note: OSS-compatible drivers are built into the kernel on Linux.
 * Compatibility libraries are available for many other Unix systems.
 * See <http://www.opensound.com/> for details and downloads.
 */

/* TODO:
 * - allow for arbitrary combinations of AFMT_S16_LE and AFMT_S16_BE
 *   for both platform and sound card, then optimise when platform
 *   and sound card use the same format (Intel and PPC use opposite
 *   byte order).
 */

#include "sq.h"
#include "SoundPlugin.h"



# warning: SOUND SUPPORT DISABLED

/* dummy definitions for systems without OSS */


void auPollForIO(void)
{
  return;
}

#define FAIL(X) { return X; }

/* sound output */

int snd_AvailableSpace(void)
     FAIL(0)  /* 8192 */
int snd_InsertSamplesFromLeadTime(int frameCount, int srcBufPtr,
				  int samplesOfLeadTime)
     FAIL(0)  /* frameCount */
int snd_PlaySamplesFromAtLength(int frameCount, int arrayIndex, int startIndex)
     FAIL(0)    /* 8192 */
int snd_PlaySilence(void)
     FAIL(0)    /* 8192 */
int snd_Start(int frameCount, int samplesPerSec, int stereo, int semaIndex)
     FAIL(0)   /* 1 */
int snd_Stop(void)
     FAIL(0)

/* sound input */

int snd_SetRecordLevel(int level)
     FAIL(0)
int snd_StartRecording(int desiredSamplesPerSec, int stereo, int semaIndex)
     FAIL(0)
int snd_StopRecording(void)
     FAIL(0)
double snd_GetRecordingSampleRate(void)
     FAIL(0)
int snd_RecordSamplesIntoAtLength(int buf, int startSliceIndex,
				  int bufferSizeInBytes)
     FAIL(0)



/*** module initialisation/shutdown ***/


typedef void (*soundPollFunction_t)(void);

extern soundPollFunction_t soundPollFunction;

int soundInit(void)
{
  soundPollFunction= auPollForIO;
  return 1;
}

int soundShutdown(void)
{
  snd_Stop();
  return 1;
}
