/******************************************************************************
FILE
    sq.m
DESCRIPTION
	This module contains stub routines. These are the low-level routines
	called by the Squeak interpreter. They're implemented by sending a
	message to the application object or the view object.
AUTHOR
	<PJB> Pascal J. Bourguignon
MODIFICATIONS
	1998/06/12 <PJB> Creation.
LEGAL
	Copyright Pascal J. Bourguignon 1998 - 1998

	This program is free software; you can redistribute it and/or
	modify it under the terms of the version 2 of the GNU General Public 
	License as published by the Free Software Foundation.
	
	This program is distributed in the hope that it will be useful, but 
	WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
	or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
	hereafter for more details.
******************************************************************************/
#import <Foundation/Foundation.h>
#import "sq.h"

// These two variable are normally defined in the translator.
int contextCacheEntries=16;
int stackCacheEntries=16;

void externalFnTest()
{
    NSLog(@"external fn called");
}

#define DEBUG

#ifdef DEBUG
# define dprintf fprintf
#else
# define dprintf
#endif

#define ZEROCOVER( fnName )  int fnName() { return 0; }
	
/* profiling */

ZEROCOVER( clearProfile )
ZEROCOVER( dumpProfile )
ZEROCOVER( startProfiling )
ZEROCOVER( stopProfiling )

//---------------- new primitives ----------------------

int ioHasDisplayDepth( int depth ) { success(false); return 0; }
int ioSetDisplayMode( int width, int height, int depth, int fullscreenFlag ) { success(false); return 0; }

ZEROCOVER( oldprimSampledSoundmixSampleCountintostartingAtleftVolrightVol )
ZEROCOVER( primOpenDefaultConfiguration )
ZEROCOVER( ioFindExternalFunctionIn )
ZEROCOVER( ioFreeModule )
ZEROCOVER( ioLoadModule )

void *socketPollFunction=NULL;
void *soundPollFunction=NULL;


#define MODULECALLBACK( moduleName ) \
ZEROCOVER( moduleName##Init )  ZEROCOVER( moduleName##Shutdown )
#define SHUTCALLBACK( moduleName ) \
ZEROCOVER( moduleName##Shutdown )
#if 0

MODULECALLBACK( asyncFile )
SHUTCALLBACK( sqFile )
SHUTCALLBACK( joystick )
MODULECALLBACK( midi )
MODULECALLBACK( serialPort )
MODULECALLBACK( socket )
MODULECALLBACK( sound )

ZEROCOVER( sqSocketListenOnPortBacklogSize )
ZEROCOVER( ioFindExternalFunctionIn )
ZEROCOVER( sqMIDIParameterGet)
ZEROCOVER( sqMIDIParameterSet)
ZEROCOVER( sqSocketAcceptFromRecvBytesSendBytesSemaIDReadSemaIDWriteSemaID)
ZEROCOVER( sqSocketCreateNetTypeSocketTypeRecvBytesSendBytesSemaIDReadSemaIDWriteSemaID)
ZEROCOVER( sqSocketGetOptionsoptionNameStartoptionNameSizereturnedValue)
ZEROCOVER( sqSocketReceiveUDPDataBufCountaddressportmoreFlag)
ZEROCOVER( sqSocketSetOptionsoptionNameStartoptionNameSizeoptionValueStartoptionValueSizereturnedValue)
ZEROCOVER( sqSockettoHostportSendDataBufCount)

ZEROCOVER( sqSocketAcceptFromRecvBytesSendBytesSemaID )
#endif

time_t convertToSqueakTime(time_t unixTime)
{
  unixTime+= localtime(&unixTime)->tm_gmtoff;
  /* Squeak epoch is Jan 1, 1901.  Unix epoch is Jan 1, 1970: 17 leap years
     and 52 non-leap years later than Squeak. */
  return unixTime + ((52*365UL + 17*366UL) * 24*60*60UL);
}

int plugInAllowAccessToFilePath(char *pathString, int pathStringLength)
{
  return true;
}


/*** sq.m / Thu Aug 27 23:21:13 MET 1998 / PJB ***/
