/* SqView.h */

#import <AppKit/AppKit.h>

#import <Squeak/SqueakInterpreter.h>

@interface SqView:NSView <SqueakDisplayServing, SqueakInputSensor>
{
    NSCursor*           cursor;

    unsigned char*      Two_to_Two;
    // These colorMaps are used only when Squeak depth is 8, to convert to
    // 2-bit, 8-bit, 16-bit or 32-bit maps.
    unsigned char*      colorMap2Gray;
    unsigned char*      colorMap8Gray;
    unsigned short*     colorMap16;
    unsigned long*      colorMap32;

    int                 lastBitsIndex; // used by updateWholeDisplay.
    int                 lastWidth;
    int                 lastHeight;
    int                 lastDepth;
    BOOL				dirty;
    id					interpreter;
    BOOL		fullScreen;
    id			originalMode;
    int			oldLevel;
    NSRect		oldWindowFrame;
    BOOL		squeakWantsFullScreen;
    id			fullScreenChecker;
}

-(void)stopInterpreter:sender;
-(void)restartInterpreter:sender;
-(void)goFullScreen:sender;
-(void)undoFullScreen:sender;

-(id)initWithFrame:(NSRect)frameRect;
-(void)startInterpreterWithImageInPath:imagePath heapSize:(int)heapSize;

// Responder methods:
-(BOOL)acceptsFirstResponder;
-(BOOL)acceptsFirstMouse;
-(BOOL)performKeyEquivalent:(NSEvent*)event;

          // SqView methods:
-(void)setCurrentMouse:(NSPoint)mousePoint;
-(NSPoint)currentMouse;
-(void)initializeTransparentCursor;
-(void)initializeSqueakColorMap;
-(BOOL)isDirty;
    /*
     May be overriden by subclasses. Set up the color maps.
     */
-(void)setColorEntry:(int)i red:(int)r green:(int)g blue:(int)b;
    /*
     May be overriden by subclasses. Set up one entry in the color maps.
     */
-(void)redisplayAfterUnhide;
    /*
     Sent from appDidUnhide:.  Do nothing.
     Subclasses may override to send updateWholeDisplay.
     */
-(void)updateWholeDisplay;

    /* display,mouse,keyboard,time i/o */

-(int)ioForceDisplayUpdate;
-(int)ioFormPrintBitsAddr:(int)bAdr width:(int)w height:(int)h depth:(int)d
                   hScale:(double)hs vScale:(double)vs landscapeFlag:(int)lf;
-(int)ioSetFullScreen:(int)fullScreen;
-(void)signalSemaphore;
-(void)point:(NSPoint*)point toSq:(int*)x:(int*)y;
-(int)characterCodeFromEvent:(NSEvent*)anEvent;

@end // SqView.

/*** SqView.h / Thu Aug 27 23:20:56 MET 1998 / PJB ***/

