/* SqView.m */

#import "SqView.h"
#import "SqViewBitmapConversion.h"
#import <ctype.h>
#import "SqClipboardHandler.h"

#define debug(a)	


@implementation SqView


idAccessor( interpreter, _setInterpreter )
idAccessor( fullScreenChecker, setFullScreenChecker )
boolAccessor( dirty, setDirty )

-(void)setInterpreter:newInterp
{
    [self _setInterpreter:newInterp];
    NSLog(@"will set display, input and event sensor");
    [newInterp setDisplay:self];
    [newInterp setEventSensor:self];
    [newInterp setInputSensor:self];
}

-(void)ioExit
{
    [[[self window] delegate] updateChangeCount:NSChangeCleared];
    [[NSApplication sharedApplication] terminate:self];
}

-(BOOL)isOpaque
{
    return YES;
}

-(void)restartInterpreter:sender
{
    [interpreter startInterpreterInSeparateThread];
}

-(void)stopInterpreter:sender
{
    [interpreter stopInterpreter];
}

-(id)initWithFrame:(NSRect)frameRect
{
    self=[super initWithFrame:frameRect];
    if(self!=0){
//        interpreter = [SqueakInterpreter interpreter];
        cursor=0;
        [self initializeSqueakColorMap];
        lastBitsIndex=0;
        lastWidth=0;
        lastHeight=0;
        lastDepth=0;
        dirty=YES;
//        [self setFullScreenChecker:[NSTimer scheduledTimerWithTimeInterval:2 target:self selector:@selector(checkFullScreen) userInfo:nil repeats:YES]];
    }
    return(self);
}//initFrame:;
	

-(BOOL)isDirty
{
    return dirty;
}

-(void)dealloc
{
    free(colorMap2Gray);
    free(colorMap8Gray);
    free(colorMap16);
    free(colorMap32);
//    [self undoFullScreen];
    [fullScreenChecker invalidate];
    [fullScreenChecker release];
    [super dealloc];
}

-(void)redisplayAfterUnhide
{
}
	
-(void)updateWholeDisplay
{
    if(lastBitsIndex!=0){
        [self ioShowDisplayBitsIndex:lastBitsIndex
                               width:lastWidth height:lastHeight depth:lastDepth
                        affectedLeft:0 affectedRight:lastWidth
                         affectedTop:0 affectedBottom:lastHeight];
    }
}	

-(int)ioForceDisplayUpdate
{
    [[self window]flushWindow];
    return 0;
}


-(int)ioFormPrintBitsAddr:(int)bAdr width:(int)w height:(int)h depth:(int)d
                   hScale:(double)hs vScale:(double)vs landscapeFlag:(int)lf
{
    return 0;
}


-(int)ioSetFullScreen:(int)wantFullScreen
{
    NSLog(@"ioSetFullScreen: %d",wantFullScreen);
//    squeakWantsFullScreen = wantFullScreen;
}

-(void)goFullScreen:sender
{
//    squeakWantsFullScreen=YES;
//    [self goFullScreen];
}

-(void)undoFullScreen:sender
{
    squeakWantsFullScreen=NO;
//    [self undoFullScreen];
}

-(void)checkFullScreen
{
    if ( squeakWantsFullScreen && !fullScreen ) {
//        [self goFullScreen];
    } else if ( !squeakWantsFullScreen && fullScreen ) {
//        [self undoFullScreen];
    }
}

-(int)ioScreenSize
{
    NSSize size = [self bounds].size;
    if ( fullScreen ) {
        size.width=1024;
        size.height=768;
    }
    return(((((int)size.width)&0xffff)<<16)
                                         |(((int)size.height)&0xffff));

}

-(int)ioScreenDepth
{
    return [self bitsPerPixel];
}

-(int)ioSetCursorBitsIndex:(int)cursorBitsIndex
                                       withMask:(int)cursorMaskIndex
                                        offsetX:(int)x offsetY:(int)y
{
    return [self ioSetCursorBitsIndex:cursorBitsIndex offsetX:x offsetY:y];
}


-(int)ioSetCursorBitsIndex:(int)cursorBitsIndex
                                        offsetX:(int)x offsetY:(int)y
{
    id pool=[[NSAutoreleasePool alloc] init];
    NSImage*            image;
    NSBitmapImageRep*   bitmap;
    NSPoint             hotSpot={-x,-y};
    unsigned char*      data;
    unsigned char*      mask;
    unsigned char*      planes[5];
    int                 i;
    
    NS_DURING
        if ( cursor != nil) {
            [cursor release];
        }
        bitmap=[[[NSBitmapImageRep alloc] initWithBitmapDataPlanes:0
                                                       pixelsWide:16
                                                       pixelsHigh:16
                                                    bitsPerSample:1
                                                  samplesPerPixel:2
                                                         hasAlpha:YES
                                                         isPlanar:YES
                                                   colorSpaceName:NSCalibratedWhiteColorSpace
                                                      bytesPerRow:2
                                                     bitsPerPixel:0] autorelease];
        [bitmap getBitmapDataPlanes:planes];
        data=planes[0];
        mask=planes[1];

        for(i=0;i<16;i++){
            unsigned int chunk;
            chunk=(unsigned)checkedLongAt(cursorBitsIndex+(4*i));
            data[i*2+0]=~(chunk>>24)&0xFF;
            data[i*2+1]=~(chunk>>16)&0xFF;
            mask[i*2+0]=~((data[i*2+0]));
            mask[i*2+1]=~((data[i*2+1]));
        }

        image=[[[NSImage alloc]init] autorelease];
        [image addRepresentation:bitmap];
        cursor=[[NSCursor alloc] initWithImage:image hotSpot:hotSpot];
        [cursor set];
    NS_HANDLER
        NSLog(@"An exception is caught in ioSetCursorBitsIndex: %@",localException);
    NS_ENDHANDLER
    [pool release];
    return 0;
}


-(int)ioShowDisplayBitsIndex:(int)bitsIndex
                       width:(int)width height:(int)height depth:(int)depth
                affectedLeft:(int)left affectedRight:(int)right
                 affectedTop:(int)top affectedBottom:(int)bottom
{
    BOOL focused=NO;
    NSBitmapImageRep* bitmap=0;
    if ( fullScreen ) {
        [self cgDirectDisplayBitsIndex:(int)bitsIndex
                                 width:(int)width height:(int)height depth:(int)depth
                          affectedLeft:(int)left affectedRight:(int)right
                           affectedTop:(int)top affectedBottom:(int)bottom];
        return 0;
    }
    bottom+=5;
    if((right<=left)||(bottom<=top)){
        return(1);
    }

    // This can happen after resizing the window:
    if(width<right){
        right=width;
    }
    if(height<bottom){
        bottom=height;
    }

    lastBitsIndex=bitsIndex;
    lastWidth=width;
    lastHeight=height;
    lastDepth=depth;
    NS_DURING
            if([self canDraw]){
                NSPoint position;
                [self lockFocus];
                focused=YES;
                //                          NSLog(@"draw %d %d %d %d %d %d",left,top,right-left,bottom-top,width,height);
                if ( ![self drawLowLevelBits:(void*)bitsIndex
                                                  width:width
                                                 height:height
                                                  depth:depth
                                                   left:left
                                                  right:right
                                                    top:top
                                                 bottom:bottom] )
                {
                    bitmap=[self computeBitmapFromBitsIndex:(void*)bitsIndex
                                                  width:width
                                                 height:height
                                                  depth:depth
                                                   left:left
                                                  right:right
                                                    top:top
                                                 bottom:bottom
                                               position:&position];
                    [bitmap drawAtPoint:position];
                    free([bitmap bitmapData]);
                }
                [[NSGraphicsContext currentContext] flushGraphics];
                [[self window]flushWindow];  // Could wait?
                [self unlockFocus];
                focused=NO;
            }
        debug(fprintf(stderr,"SqView done.\n");)
            NS_HANDLER
                NSLog(@"An exception is caught in ioShowDisplayBitsIndex");
                if(focused){
                    [[self window]flushWindow];  // Could wait?
                    [self unlockFocus];
                }
                if(bitmap!=0){
                    [bitmap release];
                    bitmap=0;
                }
                NS_ENDHANDLER
                [[self window]flushWindow];  // Could wait?
                dirty=NO;
                return(0);
}

@end
