/* SqViewCGDrawing.m created by marcel on Wed 01-Nov-2000 */

#import "SqViewCGDrawing.h"
#import <ApplicationServices/ApplicationServices.h>

@implementation SqView(CGDrawing)


-displayMode
{
    return CGDisplayCurrentMode( kCGDirectMainDisplay );
}

-(void)checkDisplay
{
    NSLog(@"display: %@",[self displayMode]);
}

-(int)bitsPerPixel
{
    return [[[self displayMode] objectForKey:@"BitsPerPixel"] intValue];
}

-(int)bitsPerSample
{
    return [[[self displayMode] objectForKey:@"BitsPerSample"] intValue];
}

static void doNothing() {}

-(BOOL)drawLowLevelBits:(void*)bitsIndex width:(int)totalWidth height:(int)totalHeight depth:(int)depth
                                        left:(int)left right:(int)right
                                        top:(int)top bottom:(int)bottom
{
    CGDataProviderRef provider=NULL;
    CGImageRef image=NULL;
    BOOL success=NO;
    int x=left;
    int y=totalHeight-bottom;
    int width=right-left;
    int height=bottom-top;
    int bytesPerRow = totalWidth * depth / 8;
    unsigned changedBitmapData = bitsIndex + bytesPerRow*top + left;
    provider=CGDataProviderCreateWithData( NULL,bitsIndex, bytesPerRow*totalHeight, doNothing );
    if ( depth == 32  ) {
        image=CGImageCreate( totalWidth,totalHeight, 8, 32, bytesPerRow, CGColorSpaceCreateDeviceRGB(),kCGImageAlphaFirst,
                                provider, NULL, 0, kCGRenderingIntentDefault );
    } else if (  depth == 16 ) {
        image=CGImageCreate( totalWidth,totalHeight, 5, 16, bytesPerRow, CGColorSpaceCreateDeviceRGB(),kCGImageAlphaNoneSkipFirst,
                                provider, NULL, 0, kCGRenderingIntentDefault );
    }
    if ( image ) {
        CGContextRef ctx = [[NSGraphicsContext currentContext] graphicsPort];
        CGContextSaveGState( ctx );
        CGContextClipToRect(ctx, CGRectMake( x,y, width,height ));
        CGContextDrawImage(ctx, CGRectMake( 0,0, totalWidth, totalHeight ), image);
        CGContextRestoreGState(ctx);
        CGContextFlush( ctx );
        CGImageRelease( image );
        success=YES;
    }
    if ( provider ) {
        CGDataProviderRelease( provider );
    }
    return success;
}                                        
-(void)drawImageUsingCG:bitmap atPoint:(NSPoint)position
{
    {

        CGImageRef image;
        float decode[6]={
            0,1,0,1,0,1
        };
        CGDataProviderRef provider=CGDataProviderCreateWithData([bitmap bitmapData],[bitmap bitmapData], [bitmap bytesPerRow]*[bitmap pixelsHigh], free);
        image = CGImageCreate([bitmap pixelsWide],
                              [bitmap pixelsHigh],
                              [bitmap bitsPerSample],
                              [bitmap bitsPerPixel],
                              [bitmap bytesPerRow],
                              CGColorSpaceCreateDeviceRGB(),
                              kCGImageAlphaNone,
                              provider,
                              decode, 0,
                              kCGRenderingIntentDefault);
        CGContextSaveGState([[NSGraphicsContext currentContext] graphicsPort]);
        CGContextClipToRect([[NSGraphicsContext currentContext] graphicsPort], CGRectMake( position.x,position.y+2,[bitmap pixelsWide],[bitmap pixelsHigh]-2));
        CGContextDrawImage([[NSGraphicsContext currentContext] graphicsPort], CGRectMake( position.x,position.y,[bitmap pixelsWide],[bitmap pixelsHigh]), image);
        CGContextRestoreGState([[NSGraphicsContext currentContext] graphicsPort]);
        [bitmap release];
        CGImageRelease(image);
        CGDataProviderRelease(provider);
    }

}

-(int)cgDirectDisplayBitsIndex:(int)bitsIndex
                       width:(int)width height:(int)height depth:(int)depth
                affectedLeft:(int)left affectedRight:(int)right
                 affectedTop:(int)top affectedBottom:(int)bottom
{
    NSPoint offset=[[self window] frame].origin;
    offset.y-=16;
    left=left & ~1;
    if ( right & 1 ) {
        right++;
    }
    if ( YES ) {
        int y,x;
        int displayWidth=CGDisplayBytesPerRow( kCGDirectMainDisplay ) / (sizeof(short));
        unsigned int *targetBits= CGDisplayAddressForPosition(kCGDirectMainDisplay, left+offset.x, top+offset.y);
        unsigned int *sourceBits=((short*)bitsIndex) + width * top + left;
 //       NSLog(@"going to draw top:%d to bottom:%d left:%d to right: %d",top,bottom,left,right);
        for ( y=top;y<bottom;y++,targetBits+=512,sourceBits+=width/2) {
            for (x=0;x<(right-left)/2;x++ ) {
                targetBits[x]=sourceBits[x];
            }
        }
    }
    return 0;
}

#define BORDERFUDGE	20

-(void)goFullScreen
{
    CFDictionaryRef mode;
    size_t desiredBitDepth = 16;
    size_t desiredWidth = 1024;
    size_t desiredHeight = 768;
    boolean_t exactMatch;

    originalMode = [CGDisplayCurrentMode( kCGDirectMainDisplay ) retain];

    mode = CGDisplayBestModeForParameters(
                                          kCGDirectMainDisplay,
                                          desiredBitDepth, desiredWidth,
                                          desiredHeight, &exactMatch );
        
    if ( NULL != mode ) {
        int level;
        /* If it is important to have an
        exact match, check exactMatch here */

        CGDisplayCapture( kCGDirectMainDisplay );
        CGDisplaySwitchToMode( kCGDirectMainDisplay, mode );
        level = CGShieldingWindowLevel();
        oldLevel = [[self window] level];
        oldWindowFrame = [[self window] frame];
        [[self window] setLevel:level];
        [[self window] setFrame:NSMakeRect( 0,-BORDERFUDGE,1024,768+BORDERFUDGE ) display:YES];
        fullScreen=YES;
    }
}


-(void)undoFullScreen
{
    if ( fullScreen ) {
        CGDisplayShowCursor( kCGDirectMainDisplay );
        CGDisplaySwitchToMode( kCGDirectMainDisplay, originalMode );
        CGDisplayRelease( kCGDirectMainDisplay );
        [[self window] setLevel:oldLevel];
        [[self window] setFrame:oldWindowFrame display:NO];
    }
    fullScreen=NO;
}


@end
