/* SqueakSoundServer.m created by marcel on Thu 15-Feb-2001 */

#import "SqueakSoundServer.h"

@implementation SqueakSoundServer


static OSStatus
callBack( AudioDeviceID inDevice, const AudioTimeStamp*	inNow, const AudioBufferList*	inInputData,
          const AudioTimeStamp*	inInputTime, AudioBufferList*		outOutputData,
          const AudioTimeStamp*	inOutputTime, void* inClientData)
{
///    NSLog(@"clientData: %x",inClientData);
    [(id)inClientData getBuffer:outOutputData->mBuffers[0].mData];
    return noErr;
}

#define	NUMBUFFERS	4
#define	BUFSIZE		8192

-(void)_dumpStatus
{
    
}

-init
{
    UInt32 outSize=4;
    int i;
    self=[super init];
    AudioHardwareGetProperty(kAudioHardwarePropertyDevices, &outSize, (void *) &audioDeviceId);
    outSize = sizeof deviceDescription;
    AudioDeviceGetProperty(audioDeviceId,0, false,
                           kAudioDevicePropertyStreamFormat,
                           &outSize,
                           (void*)&deviceDescription);
    NSLog(@"audioDeviceId: 0x%x, self %x",audioDeviceId,self);
    NSLog(@"properties: sampleRate %g, format %.4s , bytesPerPacket %d, framesPerPacket %d, bytesPerFrame %d channelsPerFrame: %d bitsPerChannel: %d",
          deviceDescription.mSampleRate,&deviceDescription.mFormatID,
          deviceDescription.mBytesPerPacket,deviceDescription.mFramesPerPacket,deviceDescription.mBytesPerFrame,
          deviceDescription.mChannelsPerFrame,deviceDescription.mBitsPerChannel);
    
    AudioDeviceAddIOProc(audioDeviceId,callBack,(void*)self);
    outSize=4;
    AudioDeviceGetProperty(audioDeviceId, 0, false, kAudioDevicePropertyBufferSize, &outSize, &bufferSize);
    NSLog(@"bufferSize: %d",bufferSize);
#if 0
    bufferSize=BUFSIZE;
    AudioDeviceSetProperty(audioDeviceId, NULL,0, false, kAudioDevicePropertyBufferSize, outSize, &bufferSize);
    AudioDeviceGetProperty(audioDeviceId, 0, false, kAudioDevicePropertyBufferSize, &outSize, &bufferSize);
    NSLog(@"bufferSize: %d",bufferSize);
#endif    
    playBuffers=[[NSMutableArray alloc] init];
    for (i=0;i<NUMBUFFERS;i++) {
        id data =[[NSMutableData alloc] initWithLength:bufferSize];
        [playBuffers addObject:data];
        [data release];
    }
    for (i=0;i<65536;i++) {
       lut[i]=(i - 32768) * (1.0/32768.0);
    }
    playIndex=0;
    fillIndex=0;
    return self;
}

static int totalCalls=0;
static int underRuns=0;

-(void)getBuffer:(void*)buffer
{
//    NSLog(@"getBuffer %x, bufferSize=%d",buffer,bufferSize);
    totalCalls++;
    if ( playIndex < fillIndex)  {
        id data = [playBuffers objectAtIndex:playIndex % NUMBUFFERS] ;
        memcpy( buffer, [data bytes], bufferSize);
//        NSLog(@"did copy");
        playIndex++;
        signalSemaphoreWithIndex( squeakSemaphore );
    } else {
        if ( underRuns++ % 100 == 0 ) {
            NSLog(@"underRuns: %d",underRuns);
        }
        memset( buffer, 0,bufferSize);
    }
}

-(int)availableSpace
{
    int space = (bufferSize*32) / (deviceDescription.mBitsPerChannel * deviceDescription.mChannelsPerFrame);
    if ( fillIndex - playIndex < NUMBUFFERS-1 ) {
        return space/(deviceDescription.mSampleRate/requestedRate);
    } else {
        return 0;
    }
}

-(int)insertSamples:(int)frameCount from:(void*)srcBufPtr leadTime:(int)samplesOfLeadTime
{
   return 0;
}

-(int)playSamplesFrom:(int)frameCount at:(int)arrayIndex length:(int)startIndex
{
    id data;
    float *out;
    short *inbuf;
    int i;
//    NSLog(@"playSamples %d frames at %d/%d",frameCount,fillIndex,playIndex);
    if ( fillIndex - playIndex < NUMBUFFERS-1 ) {
        data=[playBuffers objectAtIndex:fillIndex % NUMBUFFERS];
        out=(float*)[data bytes];
        inbuf = (short*)arrayIndex;
        if ( requestedRate == deviceDescription.mSampleRate ) {
            for (i=0;i<frameCount*2;i++) {
                *out++=*inbuf++ * 1.0/32768;
            }
        } else if ( requestedRate == deviceDescription.mSampleRate/2 ) {
            for (i=0;i<frameCount;i++) {
                out[0]=lut[*inbuf++ + 32768];
                out[2]=out[0];
                out[1]=lut[*inbuf++ + 32768];
                out[3]=out[1];
                out+=4;
            }
        }
        fillIndex++;
        if (!started && fillIndex>2) {
            OSStatus retval;
            retval = AudioDeviceStart(audioDeviceId, callBack);
            started=YES;
        }
        return frameCount;
    } else {
        return 0;
    }
}

-(int)playSilence
{
    return 0;
}



-(int)start:(int)frameCount rate:(int)samplesPerSec stereo:(int)stereo semaphore:(int)semaIndex
{
    OSStatus retval;
    UInt32 outSize=sizeof deviceDescription;

    NSLog(@"%@ %@ %d %d %d",[self class],NSStringFromSelector(_cmd),frameCount,samplesPerSec,stereo);
    squeakSemaphore=semaIndex;
    requestedRate = samplesPerSec;
#if 0
    deviceDescription.mSampleRate=requestedRate;
    retval = AudioDeviceSetProperty(audioDeviceId,NULL,0, false,
                                    kAudioDevicePropertyStreamFormat,
                           sizeof deviceDescription,
                           (void*)&deviceDescription);
    NSLog(@"set-property: %x %c%c%c%c",retval,retval>>24,(retval>>16)&0xff,
    (retval>>8)&0xff,(retval>>0)&0xff);
#endif    
    retval = AudioDeviceGetProperty(audioDeviceId,0, false,
                           kAudioDevicePropertyStreamFormat,
                           &outSize,
                           (void*)&deviceDescription);
    NSLog(@"get-property: %x %c%c%c%c",retval,retval>>24,(retval>>16)&0xff,
        (retval>>8)&0xff,(retval>>0)&0xff);
    NSLog(@"audioDeviceId: 0x%x, self %x",audioDeviceId,self);
    NSLog(@"properties: sampleRate %g, format %.4s , bytesPerPacket %d, framesPerPacket %d, bytesPerFrame %d channelsPerFrame: %d bitsPerChannel: %d",
          deviceDescription.mSampleRate,&deviceDescription.mFormatID,
          deviceDescription.mBytesPerPacket,deviceDescription.mFramesPerPacket,deviceDescription.mBytesPerFrame,
          deviceDescription.mChannelsPerFrame,deviceDescription.mBitsPerChannel);

    outSize=4;
    AudioDeviceGetProperty(audioDeviceId, 0, false, kAudioDevicePropertyBufferSize, &outSize, &bufferSize);
    NSLog(@"bufferSize: %d",bufferSize);
    return 1;
}

-(int)stop
{
    OSStatus retval;
    NSLog(@"%@ %@",[self class],NSStringFromSelector(_cmd));
    retval=AudioDeviceStop(audioDeviceId, callBack);
    NSLog(@"stopped: %d",retval);
    started=NO;
    fillIndex=0;
    playIndex=0;
    return 0;
}

@end
